%{ open Ast %}

/*Delimiters*/
%token LPAREN RPAREN LBRACE RBRACE LSQUARE RSQUARE COMMA SEMI
/* Operators */
%token PLUS MINUS TIMES DIVIDE NOT AND OR
%token EQ NEQ LT LEQ GT GEQ
%token STAR AMPERSAND
/*Types*/
%token INT VOID BOOL CHAR
/*Literals*/
%token <int> INTLITERAL
%token <string> STRINGLITERAL
%token <string> ID
%token <char> CHARLITERAL
%token TRUE FALSE
/*Control flow*/
%token IF ELSE NOELSE WHILE FOR
/*Misc*/
%token ASSIGN
%token FUN RETURN
%token EOF

%nonassoc NOELSE
%nonassoc ELSE
%nonassoc NOLSQUARE
%nonassoc LSQUARE
%right ASSIGN
%left OR
%left AND
%left EQ NEQ
%left LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE
%right NOT NEG

%start program
%type <Ast.program> program

%%

program:
  decls EOF {$1}

decls:
  /*nothing*/ {[],[]}
| decls topvdecl {($2 :: fst $1), snd $1}
| decls fdecl {fst $1, ($2 :: snd $1)}

/*these two vdecl parts are for being able to mix vdecls and fdecls at the top level (outside main)*/
topvdecl_list:
  /*nothing*/ {[]}
| topvdecl_list topvdecl {$2 :: $1}

topvdecl:
  typ ID SEMI {$1, $2}

fdecl:
  FUN typ ID LPAREN formals_opt RPAREN LBRACE fbody RBRACE
  { {return_type = $2; fname = $3; formals = $5; fbody = $8 }} /*copied from slides*/

/*depending on whether the next line is a vdecl or a stmt, add it to the correct list: f_vdecl or f_stmts*/
fbody:
  /* nothing */ { { f_vdecls = []; f_stmts = []; } }
| fbody vdecl { { f_vdecls = $1.f_vdecls @ [$2]; f_stmts = $1.f_stmts; } }
| fbody stmt { { f_vdecls = $1.f_vdecls; f_stmts = $1.f_stmts @ [$2]; } }

formals_opt:
  /*nothing*/ {[]}
| formal_list {List.rev $1}

formal_list:
  typ ID  { [$1,$2] }
| formal_list COMMA typ ID { ($3,$4) :: $1 }

typ:
  INT {Int}
| BOOL {Bool}
| CHAR { Char }
| VOID { Void }
| inttyp LSQUARE INTLITERAL RSQUARE { Matrix1DType($1, $3) }
| inttyp LSQUARE RSQUARE { Matrix1DPointer($1) }

inttyp:
  INT {Int}

stmt_list:
  /*nothing*/ {[]}
| stmt_list stmt {$2 :: $1}

vdecl:
  typ ID SEMI {$1, $2}

stmt:
| expr SEMI {Expr $1}
| RETURN SEMI {Return Noexpr}
| RETURN expr SEMI {Return $2}
| LBRACE stmt_list RBRACE {Block(List.rev $2)}
| IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
| IF LPAREN expr RPAREN stmt ELSE stmt    { If($3, $5, $7) }
| WHILE LPAREN expr RPAREN stmt { While($3, $5) }
| FOR LPAREN expr_opt SEMI expr SEMI expr_opt RPAREN stmt
   { For($3, $5, $7, $9) }

/*s--how does the list work*/
matrix_literal:
    expr { [$1] }
  | matrix_literal COMMA expr { $3 :: $1 }

expr_opt:
    /* nothing */ { Noexpr }
  | expr          { $1 }

expr:
  INTLITERAL {IntLiteral($1)}
| STRINGLITERAL { StringLiteral($1) }
| CHARLITERAL     { CharLiteral($1) }
| TRUE             { BoolLit(true) }
| FALSE            { BoolLit(false) }
| expr ASSIGN expr {Assign($1, $3)}
| expr PLUS expr  { Binop($1, Add, $3) }
| expr MINUS expr  { Binop($1, Subt, $3) }
| expr TIMES expr  { Binop($1, Mult, $3) }
| expr DIVIDE expr  { Binop($1, Div, $3) }
| expr AND    expr { Binop($1, And,   $3) }
| expr OR     expr { Binop($1, Or,    $3) }
| expr EQ     expr { Binop($1, Eq, $3) }
| expr NEQ    expr { Binop($1, Neq,   $3) }
| expr LT     expr { Binop($1, Less,  $3) }
| expr LEQ    expr { Binop($1, Leq,   $3) }
| expr GT     expr { Binop($1, Greater, $3) }
| expr GEQ    expr { Binop($1, Geq,   $3) }
| MINUS expr %prec NEG { Unop(Neg, $2) }
| NOT expr         { Unop(Not, $2) }
| LPAREN expr RPAREN {$2}
| LSQUARE matrix_literal RSQUARE { MatrixLiteral(List.rev $2) }   /*ie, [1,1,1]*/
| ID LSQUARE expr RSQUARE        { Matrix1DAccess($1, $3)}  /*ie, i[2]*/
| STAR ID                        { Matrix1DReference($2)}
| AMPERSAND ID                   { Dereference($2)}
| PLUS PLUS ID                   { PointerIncrement($3) }
| ID {Id($1)}
| ID LPAREN actuals_opt RPAREN {Call($1, $3)}

actuals_opt:
  /*nothing*/ {[]}
| actuals_list {List.rev $1}

actuals_list:
  expr {[$1]}
| actuals_list COMMA expr {$3 :: $1}
