(* WARHOL Scanner *)

{ open Parser 
	let unescape s =
    	Scanf.sscanf ("\"" ^ s ^ "\"") "%S%!" (fun x -> x)
}
let digits = ['0'-'9']
let ascii = ([' '-'!' '#'-'[' ']'-'~'])
let esc = '\\' ['\\' ''' '"' 'n' 'r' 't']
let string = '"' ( (ascii | esc)* as s ) '"'
let char = ''' ( ascii | digits ) '''

rule token = parse
[' '  '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "$" { comment lexbuf } (* Comments *)

(* Identifiers *)
| '(' { LPAREN }
| ')' { RPAREN }
| '{' { LBRACE }
| '}' { RBRACE }
| '[' { LSQUARE }
| ']' { RSQUARE }
| ',' { COMMA }
| ';' { SEMI }
| "fun" { FUN }
| "return" { RETURN }
| "int" { INT }
| '@' { STAR }
| '&' { AMPERSAND }
| '+' { PLUS }
| '-' { MINUS }
| '*' { TIMES }
| '/' { DIVIDE }
| '=' { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "!"      { NOT }
| "&&"     { AND }
| "||"     { OR }
| "if"     { IF }
| "else"   { ELSE }
| "while"  { WHILE }
| "for"    { FOR }
| "bool"   { BOOL }
| "true"   { TRUE }
| "false"  { FALSE }
| "void"   { VOID }
| ['0'-'9']+ as lxm { INTLITERAL(int_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| string { STRINGLITERAL(unescape s) }
| 
char    as lxm { CHARLITERAL(String.get lxm 1) }
| eof { EOF }

and comment = parse
"$" { token lexbuf }
| _ { comment lexbuf }
