{ open Printf }

rule token = parse
[' '  '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "$" { comment lexbuf } (* Comments *)

(* Identifiers *)

| "int" { print_string "INT " }
| "mat" { print_string "MAT " }
| "bool" { print_string "BOOL " }
| "if" { print_string "IF " }
| "else" { print_string "ELSE " }
| "for" { print_string "FOR " }
| "while" { print_string "WHILE " }
| "break" { print_string "BREAK " }
| "continue" { print_string "CONTINUE " }
| "sizeof" { print_string "SIZEOF " }
| "void" { print_string "VOID " }
| "null" { print_string "NULL " }
| "fun" { print_string "FUN " }
| "char" { print_string "CHAR " }
| "bool" { print_string "BOOL " }
| "return" { print_string "RETURN " }

(* Integers *)

| ['0'-'9']+ { print_string "LITERAL " }

(* Operators *)
| "{" { print_string "LBRACE " }
| "}" { print_string "RBRACE " }
| "==" { print_string "ISEQ " }
| "=" { print_string "ASSIGN " }
| "!=" { print_string "NEQ " }
| "<" { print_string "LTHAN " }
| ">" { print_string "GTHAN " }
| "<=" { print_string "LESSEQ " }
| ">=" { print_string "GREATEQ " }
| "+" { print_string "ADD " }
| "-" { print_string "SUB " }
| "*" { print_string "MULT " }
| "/" { print_string "DIV " }
| "!" { print_string "NEGATE " }
| "-" { print_string "NEG " }
| "!=" { print_string "NEQ " }
| "&&" { print_string "AND " }
| "||" { print_string "OR " }
| "(" { print_string "LPAREN " }
| ")" { print_string "RPAREN " }


(* Identifiers *)

| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* { print_string "ID "}

(* Characters -- work on this *)

| [''']* { print_string "CHAR "}

(* End of line *)

| eof { print_string "EOF" }
| _ as char { raise (Failure("illegal character " ^
Char.escaped char)) }

(* End Comment *)

and comment = parse
"$" { token lexbuf }
| _ { comment lexbuf }


(* Test *)

{
	let main () =
    let lexbuf = Lexing.from_channel stdin in (* take stdin *)
    try
        while true do
            ignore (token lexbuf)
        done
    with _ -> print_string "invalid_token(test is over)\n"
  let _ = Printexc.print main ()
}
